@@ -43,11 +43,14 @@
 
 namespace aria2 {
 
-AbstractBtMessage::AbstractBtMessage():sendingInProgress(false),
-				       invalidate(false),
-				       uploading(false),
-				       cuid(0),
-				       logger(LogFactory::getInstance())
+AbstractBtMessage::AbstractBtMessage(uint8_t id, const std::string& name):
+  BtMessage(id),
+  sendingInProgress(false),
+  invalidate(false),
+  uploading(false),
+  cuid(0),
+  _name(name),
+  logger(LogFactory::getInstance())
 {}
 
 AbstractBtMessage::~AbstractBtMessage() {}
@@ -130,4 +133,9 @@ void AbstractBtMessage::setBtRequestFactory(const WeakHandle<BtRequestFactory>&
   this->requestFactory = factory;
 }
 
+const std::string& AbstractBtMessage::getName() const
+{
+  return _name;
+}
+
 } // namespace aria2
