@@ -35,6 +35,9 @@
 #include "AbstractSingleDiskAdaptor.h"
 #include "File.h"
 #include "SingleFileAllocationIterator.h"
+#ifdef HAVE_POSIX_FALLOCATE
+# include "FallocFileAllocationIterator.h"
+#endif // HAVE_POSIX_FALLOCATE
 #include "DiskWriter.h"
 
 namespace aria2 {
@@ -90,10 +93,19 @@ void AbstractSingleDiskAdaptor::truncate(uint64_t length)
 
 FileAllocationIteratorHandle AbstractSingleDiskAdaptor::fileAllocationIterator()
 {
-  SingleFileAllocationIteratorHandle h
-    (new SingleFileAllocationIterator(this, size(), totalLength));
-  h->init();
-  return h;
+#ifdef HAVE_POSIX_FALLOCATE
+  if(_fallocate) {
+    SharedHandle<FallocFileAllocationIterator> h
+      (new FallocFileAllocationIterator(this, size(), totalLength));
+    return h;
+  } else
+#endif // HAVE_POSIX_FALLOCATE
+    {
+      SingleFileAllocationIteratorHandle h
+	(new SingleFileAllocationIterator(this, size(), totalLength));
+      h->init();
+      return h;
+    }
 }
 
 void AbstractSingleDiskAdaptor::enableDirectIO()
@@ -111,6 +123,16 @@ bool AbstractSingleDiskAdaptor::directIOAllowed() const
   return diskWriter->directIOAllowed();
 }
 
+void AbstractSingleDiskAdaptor::enableReadOnly()
+{
+  diskWriter->enableReadOnly();
+}
+
+void AbstractSingleDiskAdaptor::disableReadOnly()
+{
+  diskWriter->disableReadOnly();
+}
+
 void AbstractSingleDiskAdaptor::cutTrailingGarbage()
 {
   if(File(getFilePath()).size() > totalLength) {
