@@ -38,6 +38,7 @@
 #include <ostream>
 #include <functional>
 #include <algorithm>
+#include <vector>
 
 #include "DlAbortEx.h"
 #include "Util.h"
@@ -149,17 +150,22 @@ void DefaultBtContext::extractFileEntries(const bencode::BDE& infoDict,
       if(!pathList.isList() || pathList.empty()) {
 	throw DlAbortEx("Path is empty.");
       }
-      std::string path;
-      for(size_t i = 0; i < pathList.size()-1; ++i) {
-	path += pathList[i].s()+"/";
-
-      }
-      path += pathList[pathList.size()-1].s();
+      
+      std::vector<std::string> pathelem(pathList.size());
+      std::transform(pathList.listBegin(), pathList.listEnd(), pathelem.begin(),
+		     std::mem_fun_ref(&bencode::BDE::s));
+      std::string path =
+	name+"/"+Util::joinPath(pathelem.begin(), pathelem.end());
+      // Split path with '/' again because each pathList element can
+      // contain "/" inside.
+      std::deque<std::string> elements;
+      Util::slice(elements, path, '/');
+      path = Util::joinPath(elements.begin(), elements.end());
 
       std::deque<std::string> uris;
       std::transform(urlList.begin(), urlList.end(), std::back_inserter(uris),
-		     std::bind2nd(std::plus<std::string>(), "/"+name+"/"+path));
-      FileEntryHandle fileEntry(new FileEntry(path, fileLengthData.i(),
+		     std::bind2nd(std::plus<std::string>(), "/"+path));
+      FileEntryHandle fileEntry(new FileEntry(_dir+"/"+path, fileLengthData.i(),
 					      offset, uris));
       fileEntries.push_back(fileEntry);
       offset += fileEntry->getLength();
@@ -174,7 +180,12 @@ void DefaultBtContext::extractFileEntries(const bencode::BDE& infoDict,
 				     BtContext::C_LENGTH.c_str()).str());      
     }
     totalLength = lengthData.i();
-    FileEntryHandle fileEntry(new FileEntry(name, totalLength, 0, urlList));
+    // Slice path by '/' just in case nasty ".." is included in name
+    std::deque<std::string> pathelems;
+    Util::slice(pathelems, name, '/');
+    SharedHandle<FileEntry> fileEntry
+      (new FileEntry(_dir+"/"+Util::joinPath(pathelems.begin(),pathelems.end()),
+		     totalLength, 0, urlList));
     fileEntries.push_back(fileEntry);
   }
 }
@@ -475,4 +486,15 @@ void DefaultBtContext::setInfoHash(const unsigned char* infoHash)
   memcpy(this->infoHash, infoHash, sizeof(this->infoHash));
 }
 
+void DefaultBtContext::setFilePathWithIndex
+(size_t index, const std::string& path)
+{
+  if(0 < index && index <= fileEntries.size()) {
+    fileEntries[index-1]->setPath(path);
+  } else {
+    throw DlAbortEx(StringFormat("No such file with index=%u",
+				 static_cast<unsigned int>(index)).str());
+  }
+}
+
 } // namespace aria2
