@@ -66,6 +66,7 @@
 #include "LogFactory.h"
 #include "StringFormat.h"
 #include "RequestGroup.h"
+#include "RequestGroupMan.h"
 
 namespace aria2 {
 
@@ -77,7 +78,6 @@ DefaultBtInteractive::DefaultBtInteractive
   logger(LogFactory::getInstance()),
   allowedFastSetSize(10),
   keepAliveInterval(120),
-  maxDownloadSpeedLimit(0),
   _utPexEnabled(false),
   _dhtEnabled(false),
   _numReceivedMessage(0)
@@ -237,11 +237,9 @@ void DefaultBtInteractive::sendKeepAlive() {
 size_t DefaultBtInteractive::receiveMessages() {
   size_t msgcount = 0;
   for(int i = 0; i < 50; i++) {
-    if(maxDownloadSpeedLimit > 0) {
-      TransferStat stat = _btContext->getOwnerRequestGroup()->calculateStat();
-      if(maxDownloadSpeedLimit < stat.downloadSpeed) {
-	break;
-      }
+    if(_requestGroupMan->doesOverallDownloadSpeedExceed() ||
+       _btContext->getOwnerRequestGroup()->doesDownloadSpeedExceed()) {
+      break;
     }
     BtMessageHandle message = btMessageReceiver->receiveMessage();
     if(message.isNull()) {
@@ -267,8 +265,10 @@ size_t DefaultBtInteractive::receiveMessages() {
 	floodingStat.incChokeUnchokeCount();
       }
       break;
-    case BtRequestMessage::ID:
     case BtPieceMessage::ID:
+      _peerStorage->updateTransferStatFor(peer);
+      // pass through
+    case BtRequestMessage::ID:
       inactiveCheckPoint.reset();
       break;
     }
@@ -438,8 +438,10 @@ void DefaultBtInteractive::doInteractionProcessing() {
 
   detectMessageFlooding();
 
-  dispatcher->checkRequestSlotAndDoNecessaryThing();
-
+  if(_perSecCheckPoint.elapsed(1)) {
+    _perSecCheckPoint.reset();
+    dispatcher->checkRequestSlotAndDoNecessaryThing();
+  }
   checkHave();
 
   sendKeepAlive();
@@ -544,4 +546,10 @@ void DefaultBtInteractive::setBtMessageFactory
   this->messageFactory = factory;
 }
 
+void DefaultBtInteractive::setRequestGroupMan
+(const WeakHandle<RequestGroupMan>& rgman)
+{
+  _requestGroupMan = rgman;
+}
+
 } // namespace aria2
