@@ -37,6 +37,10 @@
 
 #include "PeerStorage.h"
 
+#include <map>
+
+#include "TimeA2.h"
+
 namespace aria2 {
 
 class BtContext;
@@ -61,6 +65,12 @@ private:
   BtSeederStateChoke* _seederStateChoke;
   BtLeecherStateChoke* _leecherStateChoke;
 
+  std::map<std::string, TransferStat> _peerTransferStatMap;
+
+  Time _lastTransferStatMapUpdated;
+
+  TransferStat _cachedTransferStat;
+
   bool isPeerAlreadyAdded(const SharedHandle<Peer>& peer);
 public:
   DefaultPeerStorage(const SharedHandle<BtContext>& btContext,
@@ -86,6 +96,8 @@ public:
 
   virtual TransferStat calculateStat();
 
+  virtual void updateTransferStatFor(const SharedHandle<Peer>& peer);
+
   virtual void returnPeer(const SharedHandle<Peer>& peer);
 
   virtual bool chokeRoundIntervalElapsed();
