@@ -45,7 +45,6 @@
 #include "prefs.h"
 #include "DirectDiskAdaptor.h"
 #include "MultiDiskAdaptor.h"
-#include "CopyDiskAdaptor.h"
 #include "DiskWriter.h"
 #include "BitfieldManFactory.h"
 #include "BitfieldMan.h"
@@ -58,6 +57,7 @@
 #include "Option.h"
 #include "StringFormat.h"
 #include "RarestPieceSelector.h"
+#include "array_fun.h"
 
 namespace aria2 {
 
@@ -98,18 +98,21 @@ bool DefaultPieceStorage::isEndGame()
 
 bool DefaultPieceStorage::getMissingPieceIndex(size_t& index,
 					       const unsigned char* bitfield,
-					       size_t& length)
+					       size_t length)
 {
-  std::deque<size_t> indexes;
+  const size_t mislen = bitfieldMan->getBitfieldLength();
+  array_ptr<unsigned char> misbitfield(new unsigned char[mislen]);
   bool r;
   if(isEndGame()) {
-    r = bitfieldMan->getAllMissingIndexes(indexes, bitfield, length);
+    r = bitfieldMan->getAllMissingIndexes(misbitfield, mislen,
+					  bitfield, length);
   } else {
-    r = bitfieldMan->getAllMissingUnusedIndexes(indexes, bitfield, length);
+    r = bitfieldMan->getAllMissingUnusedIndexes(misbitfield, mislen,
+						bitfield, length);
   }
   if(r) {
     // We assume indexes is sorted using comparator less.
-    return _pieceSelector->select(index, indexes);
+    return _pieceSelector->select(index, misbitfield,bitfieldMan->countBlock());
   } else {
     return false;
   }
@@ -490,28 +493,20 @@ void DefaultPieceStorage::initStorage()
     this->diskAdaptor = directDiskAdaptor;
   } else {
     // file mode == DownloadContext::MULTI
-    if(option->getAsBool(PREF_DIRECT_FILE_MAPPING)) {
-      logger->debug("Instantiating MultiDiskAdaptor");
-      MultiDiskAdaptorHandle multiDiskAdaptor(new MultiDiskAdaptor());
-      multiDiskAdaptor->setDirectIOAllowed(option->getAsBool(PREF_ENABLE_DIRECT_IO));
-      multiDiskAdaptor->setPieceLength(downloadContext->getPieceLength());
-      multiDiskAdaptor->setTopDir(downloadContext->getName());
-      multiDiskAdaptor->setMaxOpenFiles(option->getAsInt(PREF_BT_MAX_OPEN_FILES));
-      this->diskAdaptor = multiDiskAdaptor;
-    } else {
-      logger->debug("Instantiating CopyDiskAdaptor");
-      DiskWriterHandle writer = _diskWriterFactory->newDiskWriter();
-      writer->setDirectIOAllowed(option->getAsBool(PREF_ENABLE_DIRECT_IO));
-      CopyDiskAdaptorHandle copyDiskAdaptor(new CopyDiskAdaptor());
-      copyDiskAdaptor->setDiskWriter(writer);
-      copyDiskAdaptor->setTempFilename(downloadContext->getName()+".a2tmp");
-      copyDiskAdaptor->setTotalLength(downloadContext->getTotalLength());
-      copyDiskAdaptor->setTopDir(downloadContext->getName());
-      this->diskAdaptor = copyDiskAdaptor;
-    }
+    logger->debug("Instantiating MultiDiskAdaptor");
+    MultiDiskAdaptorHandle multiDiskAdaptor(new MultiDiskAdaptor());
+    multiDiskAdaptor->setDirectIOAllowed(option->getAsBool(PREF_ENABLE_DIRECT_IO));
+    multiDiskAdaptor->setPieceLength(downloadContext->getPieceLength());
+    multiDiskAdaptor->setMaxOpenFiles(option->getAsInt(PREF_BT_MAX_OPEN_FILES));
+    this->diskAdaptor = multiDiskAdaptor;
   }
   diskAdaptor->setStoreDir(downloadContext->getDir());
   diskAdaptor->setFileEntries(downloadContext->getFileEntries());
+#ifdef HAVE_POSIX_FALLOCATE
+  if(option->get(PREF_FILE_ALLOCATION) == V_FALLOC) {
+    diskAdaptor->enableFallocate();
+  }
+#endif // HAVE_POSIX_FALLOCATE
 }
 
 void DefaultPieceStorage::setBitfield(const unsigned char* bitfield,
