@@ -50,6 +50,9 @@ class DiskAdaptor:public BinaryStream {
 protected:
   std::string storeDir;
   std::deque<SharedHandle<FileEntry> > fileEntries;
+#ifdef HAVE_POSIX_FALLOCATE
+  bool _fallocate;
+#endif // HAVE_POSIX_FALLOCATE
   Logger* logger;
 public:
   DiskAdaptor();
@@ -67,13 +70,8 @@ public:
 
   virtual bool fileExists() = 0;
 
-  virtual std::string getFilePath() = 0;
-
   virtual uint64_t size() = 0;
 
-  // optional behavior
-  virtual void truncate(uint64_t length) {}
-
   void setFileEntries(const std::deque<SharedHandle<FileEntry> >& fileEntries);
 
   SharedHandle<FileEntry> getFileEntryFromPath(const std::string& fileEntryPath) const;
@@ -98,6 +96,10 @@ public:
 
   virtual void disableDirectIO() {}
 
+  virtual void enableReadOnly() {}
+
+  virtual void disableReadOnly() {}
+
   // Assumed each file length is stored in fileEntries or DiskAdaptor knows it.
   // If each actual file's length is larger than that, truncate file to that
   // length.
@@ -108,6 +110,23 @@ public:
   // Returns the number of files, the actime and modtime of which are
   // successfully changed.
   virtual size_t utime(const Time& actime, const Time& modtime) = 0;
+
+#ifdef HAVE_POSIX_FALLOCATE
+  void enableFallocate()
+  {
+    _fallocate = true;
+  }
+
+  void disableFallocate()
+  {
+    _fallocate = false;
+  }
+
+  bool doesFallocate() const
+  {
+    return _fallocate;
+  }
+#endif // HAVE_POSIX_FALLOCATE
 };
 
 typedef SharedHandle<DiskAdaptor> DiskAdaptorHandle;
