@@ -129,7 +129,7 @@ bool HttpResponseCommand::executeInternal()
     SingleFileDownloadContextHandle dctx =
       dynamic_pointer_cast<SingleFileDownloadContext>(_requestGroup->getDownloadContext());
     dctx->setTotalLength(totalLength);
-    dctx->setFilename(httpResponse->determinFilename());
+    dctx->setFilename(dctx->getDir()+"/"+httpResponse->determinFilename());
     dctx->setContentType(httpResponse->getContentType());
     _requestGroup->preDownloadProcessing();
     if(e->_requestGroupMan->isSameFileBeingDownloaded(_requestGroup)) {
@@ -197,6 +197,11 @@ bool HttpResponseCommand::handleDefaultEncoding(const HttpResponseHandle& httpRe
   HttpRequestHandle httpRequest = httpResponse->getHttpRequest();
   _requestGroup->initPieceStorage();
 
+  if(e->option->getAsBool(PREF_DRY_RUN)) {
+    onDryRunFileFound();
+    return true;
+  }
+
   BtProgressInfoFileHandle infoFile(new DefaultBtProgressInfoFile(_requestGroup->getDownloadContext(), _requestGroup->getPieceStorage(), e->option));
   if(!infoFile->exists() && _requestGroup->downloadFinishedByFileLength()) {
     return true;
@@ -270,6 +275,13 @@ static SharedHandle<Decoder> getContentEncodingDecoder
 bool HttpResponseCommand::handleOtherEncoding(const HttpResponseHandle& httpResponse) {
   // We assume that RequestGroup::getTotalLength() == 0 here
   HttpRequestHandle httpRequest = httpResponse->getHttpRequest();
+
+  if(e->option->getAsBool(PREF_DRY_RUN)) {
+    _requestGroup->initPieceStorage();
+    onDryRunFileFound();
+    return true;
+  }
+
   if(req->getMethod() == Request::METHOD_HEAD) {
     poolConnection();
     req->setMethod(Request::METHOD_GET);
@@ -334,8 +346,6 @@ HttpDownloadCommand* HttpResponseCommand::createHttpDownloadCommand
   HttpDownloadCommand* command =
     new HttpDownloadCommand(cuid, req, _requestGroup,
 			    httpResponse, httpConnection, e, socket);
-  command->setMaxDownloadSpeedLimit
-    (e->option->getAsInt(PREF_MAX_DOWNLOAD_LIMIT));
   command->setStartupIdleTime(e->option->getAsInt(PREF_STARTUP_IDLE_TIME));
   command->setLowestDownloadSpeedLimit
     (e->option->getAsInt(PREF_LOWEST_SPEED_LIMIT));
@@ -361,4 +371,10 @@ void HttpResponseCommand::poolConnection()
   }
 }
 
+void HttpResponseCommand::onDryRunFileFound()
+{
+  _requestGroup->getPieceStorage()->markAllPiecesDone();
+  poolConnection();
+}
+
 } // namespace aria2
