@@ -203,6 +203,9 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
 				    TEXT_FILE_ALLOCATION,
 				    V_PREALLOC,
 				    V_NONE, V_PREALLOC,
+#ifdef HAVE_POSIX_FALLOCATE
+				    V_FALLOC,
+#endif // HAVE_POSIX_FALLOCATE
 				    'a'));
     op->addTag(TAG_BASIC);
     handlers.push_back(op);
@@ -265,7 +268,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
 				   (PREF_MAX_CONCURRENT_DOWNLOADS,
 				    TEXT_MAX_CONCURRENT_DOWNLOADS,
 				    "5",
-				    1, 45,
+				    1, -1,
 				    'j'));
     op->addTag(TAG_BASIC);
     handlers.push_back(op);
@@ -281,6 +284,17 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->addTag(TAG_HTTP);
     handlers.push_back(op);
   }
+  {
+    SharedHandle<OptionHandler> op(new UnitNumberOptionHandler
+				   (PREF_MAX_OVERALL_DOWNLOAD_LIMIT,
+				    TEXT_MAX_OVERALL_DOWNLOAD_LIMIT,
+				    "0",
+				    0));
+    op->addTag(TAG_BITTORRENT);
+    op->addTag(TAG_FTP);
+    op->addTag(TAG_HTTP);
+    handlers.push_back(op);
+  }
   {
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
 				   (PREF_NO_CONF,
@@ -358,6 +372,16 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->addTag(TAG_HTTP);
     handlers.push_back(op);
   }
+  {
+    SharedHandle<OptionHandler> op(new BooleanOptionHandler
+				   (PREF_DRY_RUN,
+				    TEXT_DRY_RUN,
+				    V_FALSE,
+				    OptionHandler::OPT_ARG));
+    op->addTag(TAG_FTP);
+    op->addTag(TAG_HTTP);
+    handlers.push_back(op);
+  }
   {
     SharedHandle<OptionHandler> op(new UnitNumberOptionHandler
 				   (PREF_LOWEST_SPEED_LIMIT,
@@ -413,6 +437,15 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->addTag(TAG_HTTP);
     handlers.push_back(op);
   }
+  {
+    SharedHandle<OptionHandler> op(new BooleanOptionHandler
+				   (PREF_RESET_URI,
+				    NO_DESCRIPTION,
+				    V_FALSE,
+				    OptionHandler::OPT_ARG));
+    op->hide();
+    handlers.push_back(op);
+  }
   {
     SharedHandle<OptionHandler> op(new NumberOptionHandler
 				   (PREF_RETRY_WAIT,
@@ -423,6 +456,14 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->addTag(TAG_HTTP);
     handlers.push_back(op);
   }
+  {
+    SharedHandle<OptionHandler> op(new BooleanOptionHandler
+				   (PREF_REUSE_URI,
+				    NO_DESCRIPTION,
+				    V_TRUE));
+    op->hide();
+    handlers.push_back(op);
+  }
   {
     SharedHandle<UnitNumberOptionHandler> op(new UnitNumberOptionHandler
 					     (PREF_SEGMENT_SIZE,
@@ -499,7 +540,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     SharedHandle<OptionHandler> op(new ParameterOptionHandler
 				   (PREF_URI_SELECTOR,
 				    TEXT_URI_SELECTOR,
-				    V_INORDER,
+				    V_FEEDBACK,
 				    std::deque<std::string>
 				    (&params[0], &params[arrayLength(params)])));
     op->addTag(TAG_FTP);
@@ -561,6 +602,15 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->addTag(TAG_HTTP);
     handlers.push_back(op);
   }
+  {
+    SharedHandle<OptionHandler> op(new BooleanOptionHandler
+				   (PREF_HTTP_AUTH_CHALLENGE,
+				    TEXT_HTTP_AUTH_CHALLENGE,
+				    V_FALSE,
+				    OptionHandler::OPT_ARG));
+    op->addTag(TAG_HTTP);
+    handlers.push_back(op);
+  }
   {
     SharedHandle<OptionHandler> op(new ParameterOptionHandler
 				   (PREF_HTTP_AUTH_SCHEME,
@@ -884,10 +934,11 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     handlers.push_back(op);
   }
   {
-    SharedHandle<OptionHandler> op(new BooleanOptionHandler
-				   (PREF_DIRECT_FILE_MAPPING,
-				    TEXT_DIRECT_FILE_MAPPING,
-				    V_TRUE));
+    SharedHandle<NumberOptionHandler> op(new NumberOptionHandler
+					 (PREF_BT_TRACKER_INTERVAL,
+					  TEXT_BT_TRACKER_INTERVAL,
+					  "0",
+					  0));
     op->addTag(TAG_BITTORRENT);
     handlers.push_back(op);
   }
@@ -948,6 +999,14 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->addTag(TAG_BITTORRENT);
     handlers.push_back(op);
   }
+  {
+    SharedHandle<OptionHandler> op(new IndexOutOptionHandler
+				   (PREF_INDEX_OUT,
+				    TEXT_INDEX_OUT,
+				    'O'));
+    op->addTag(TAG_BITTORRENT);
+    handlers.push_back(op);
+  }
   {
     SharedHandle<OptionHandler> op(new IntegerRangeOptionHandler
 				   (PREF_LISTEN_PORT,
