@@ -129,22 +129,21 @@ RequestGroup::RequestGroup(const Option* option,
   _timeout(option->getAsInt(PREF_TIMEOUT)),
   _maxTries(option->getAsInt(PREF_MAX_TRIES)),
   _inMemoryDownload(false),
+  _maxDownloadSpeedLimit(option->getAsInt(PREF_MAX_DOWNLOAD_LIMIT)),
+  _maxUploadSpeedLimit(option->getAsInt(PREF_MAX_UPLOAD_LIMIT)),
   _option(option),
   _logger(LogFactory::getInstance())
 {
-  if(_option->get(PREF_FILE_ALLOCATION) == V_PREALLOC) {
-    _fileAllocationEnabled = true;
-  } else {
-    _fileAllocationEnabled = false;
-  }
+  _fileAllocationEnabled = _option->get(PREF_FILE_ALLOCATION) != V_NONE;
   // Add types to be sent as a Accept header value here.
   // It would be good to put this value in Option so that user can tweak
   // and add this list.
   // ACCEPT_METALINK is used for `transparent metalink'.
   addAcceptType(ACCEPT_METALINK);
-
-  initializePreDownloadHandler();
-  initializePostDownloadHandler();
+  if(!_option->getAsBool(PREF_DRY_RUN)) {
+    initializePreDownloadHandler();
+    initializePostDownloadHandler();
+  }
 }
 
 RequestGroup::~RequestGroup() {}
@@ -202,6 +201,17 @@ void RequestGroup::createInitialCommand(std::deque<Command*>& commands,
   {
     BtContextHandle btContext = dynamic_pointer_cast<BtContext>(_downloadContext);
     if(!btContext.isNull()) {
+      if(_option->getAsBool(PREF_DRY_RUN)) {
+	throw DownloadFailureException
+	  ("Cancel BitTorrent download in dry-run context.");
+      }
+      SharedHandle<BtRegistry> btRegistry = e->getBtRegistry();
+      if(!btRegistry->getBtContext(btContext->getInfoHashAsString()).isNull()) {
+	throw DownloadFailureException
+	  (StringFormat("InfoHash %s is already registered.",
+			btContext->getInfoHashAsString().c_str()).str());
+      }
+
       if(e->_requestGroupMan->isSameFileBeingDownloaded(this)) {
 	throw DownloadFailureException
 	  (StringFormat(EX_DUPLICATE_FILE_DOWNLOAD,
@@ -215,7 +225,7 @@ void RequestGroup::createInitialCommand(std::deque<Command*>& commands,
 	_logger->debug("Clearing http/ftp URIs because the current implementation does not allow integrating multi-file torrent and http/ftp.");
 	_uris.clear();
 
-	_pieceStorage->setFileFilter(Util::parseIntRange(_option->get(PREF_SELECT_FILE)));
+	_pieceStorage->setFileFilter(btContext->getFileFilter());
       }
       
       SharedHandle<DefaultBtProgressInfoFile>
@@ -223,8 +233,6 @@ void RequestGroup::createInitialCommand(std::deque<Command*>& commands,
 						       _pieceStorage,
 						       _option));
       
-      SharedHandle<BtRegistry> btRegistry = e->getBtRegistry();
-
       btRegistry->registerBtContext(btContext->getInfoHashAsString(),
 				    btContext);
       btRegistry->registerPieceStorage(btContext->getInfoHashAsString(),
@@ -255,6 +263,8 @@ void RequestGroup::createInitialCommand(std::deque<Command*>& commands,
       btAnnounce->setBtRuntime(btRuntime);
       btAnnounce->setPieceStorage(_pieceStorage);
       btAnnounce->setPeerStorage(peerStorage);
+      btAnnounce->setUserDefinedInterval
+	(_option->getAsInt(PREF_BT_TRACKER_INTERVAL));
       btRegistry->registerBtAnnounce(btContext->getInfoHashAsString(),
 				     btAnnounce);
       btAnnounce->shuffleAnnounce();
@@ -264,8 +274,11 @@ void RequestGroup::createInitialCommand(std::deque<Command*>& commands,
 	progressInfoFile->removeFile();
 	_logger->notice(MSG_REMOVED_DEFUNCT_CONTROL_FILE,
 			progressInfoFile->getFilename().c_str(),
-			_pieceStorage->getDiskAdaptor()->getFilePath().c_str());
+			getFilePath().c_str());
       }
+      // First, make DiskAdaptor read-only mode.
+      _pieceStorage->getDiskAdaptor()->enableReadOnly();
+
       // Call Load, Save and file allocation command here
       if(progressInfoFile->exists()) {
 	// load .aria2 file if it exists.
@@ -280,7 +293,7 @@ void RequestGroup::createInitialCommand(std::deque<Command*>& commands,
 	    throw DownloadFailureException
 	      (StringFormat
 	       (MSG_FILE_ALREADY_EXISTS,
-		_pieceStorage->getDiskAdaptor()->getFilePath().c_str()).str());
+		getFilePath().c_str()).str());
 	  } else {
 	    _pieceStorage->getDiskAdaptor()->openFile();
 	  }
@@ -323,7 +336,8 @@ void RequestGroup::createInitialCommand(std::deque<Command*>& commands,
 #endif // ENABLE_BITTORRENT
   // TODO I assume here when totallength is set to DownloadContext and it is
   // not 0, then filepath is also set DownloadContext correctly....
-  if(_downloadContext->getTotalLength() == 0) {
+  if(_option->getAsBool(PREF_DRY_RUN) ||
+     _downloadContext->getTotalLength() == 0) {
     createNextCommand(commands, e, 1, method);
   }else {
     if(e->_requestGroupMan->isSameFileBeingDownloaded(this)) {
@@ -427,7 +441,7 @@ void RequestGroup::loadAndOpenFile(const BtProgressInfoFileHandle& progressInfoF
       progressInfoFile->removeFile();
       _logger->notice(MSG_REMOVED_DEFUNCT_CONTROL_FILE,
 		      progressInfoFile->getFilename().c_str(),
-		      _pieceStorage->getDiskAdaptor()->getFilePath().c_str());
+		      getFilePath().c_str());
     }
     while(1) {
       if(progressInfoFile->exists()) {
@@ -509,11 +523,10 @@ bool RequestGroup::tryAutoFileRenaming()
 
   for(unsigned int i = 1; i < 10000; ++i) {
     File newfile(filepath+"."+Util::uitos(i));
-    std::string newFilename = newfile.getBasename();
-    tempCtx->setUFilename(newFilename);
+    tempCtx->setUFilename(newfile.getPath());
     tempInfoFile.updateFilename();
     if(!newfile.exists() || (newfile.exists() && tempInfoFile.exists())) {
-      ctx->setUFilename(newFilename);
+      ctx->setUFilename(newfile.getPath());
       return true;
     }
   }
@@ -547,6 +560,40 @@ void RequestGroup::createNextCommand(std::deque<Command*>& commands,
 				     unsigned int numCommand,
 				     const std::string& method)
 {
+  if(_option->getAsBool(PREF_REUSE_URI) && _uris.empty()) {
+    std::deque<std::string> uris = _spentUris;
+    std::sort(uris.begin(), uris.end());
+    uris.erase(std::unique(uris.begin(), uris.end()), uris.end());
+
+    std::deque<std::string> errorUris(_uriResults.size());
+    std::transform(_uriResults.begin(), _uriResults.end(),
+		   errorUris.begin(), std::mem_fun_ref(&URIResult::getURI));
+    std::sort(errorUris.begin(), errorUris.end());
+    errorUris.erase(std::unique(errorUris.begin(), errorUris.end()),
+		    errorUris.end());
+     
+    std::deque<std::string> reusableURIs;
+    std::set_difference(uris.begin(), uris.end(),
+			errorUris.begin(), errorUris.end(),
+			std::back_inserter(reusableURIs));
+    size_t ininum = reusableURIs.size();
+    _logger->debug("Found %u reusable URIs",
+		   static_cast<unsigned int>(ininum));
+    // Reuse at least _numConcurrentCommand URIs here to avoid to
+    // run this process repeatedly.
+    if(ininum > 0 && ininum < _numConcurrentCommand) {
+      _logger->debug("fewer than _numConcurrentCommand=%u",
+		     _numConcurrentCommand);
+      for(size_t i = 0; i < _numConcurrentCommand/ininum; ++i) {
+	_uris.insert(_uris.end(), reusableURIs.begin(), reusableURIs.end());
+      }
+      _uris.insert(_uris.end(), reusableURIs.begin(),
+		   reusableURIs.begin()+(_numConcurrentCommand%ininum));
+      _logger->debug("Duplication complete: now %u URIs for reuse",
+		     static_cast<unsigned int>(_uris.size()));
+    }
+  }
+
   std::deque<std::string> pendingURIs;
   for(; numCommand--; ) {    
     std::string uri = _uriSelector->select(_uris);
@@ -1177,4 +1224,16 @@ unsigned int RequestGroup::getMaxTries() const
   return _maxTries;
 }
 
+bool RequestGroup::doesDownloadSpeedExceed()
+{
+  return _maxDownloadSpeedLimit > 0 &&
+    _maxDownloadSpeedLimit < calculateStat().getDownloadSpeed();
+}
+
+bool RequestGroup::doesUploadSpeedExceed()
+{
+  return _maxUploadSpeedLimit > 0 &&
+    _maxUploadSpeedLimit < calculateStat().getUploadSpeed();
+}
+
 } // namespace aria2
