@@ -55,7 +55,7 @@
 #include "ServerStat.h"
 #include "PeerStat.h"
 #include "SegmentMan.h"
-#include "ServerStatURISelector.h"
+#include "FeedbackURISelector.h"
 #include "InOrderURISelector.h"
 #include "AdaptiveURISelector.h"
 #include "Option.h"
@@ -63,6 +63,7 @@
 #include "File.h"
 #include "Util.h"
 #include "Command.h"
+#include "FileEntry.h"
 
 namespace aria2 {
 
@@ -74,7 +75,11 @@ RequestGroupMan::RequestGroupMan(const RequestGroups& requestGroups,
   _maxSimultaneousDownloads(maxSimultaneousDownloads),
   _gidCounter(0),
   _option(option),
-  _serverStatMan(new ServerStatMan()) {}
+  _serverStatMan(new ServerStatMan()),
+  _maxOverallDownloadSpeedLimit
+  (option->getAsInt(PREF_MAX_OVERALL_DOWNLOAD_LIMIT)),
+  _maxOverallUploadSpeedLimit(option->getAsInt(PREF_MAX_OVERALL_UPLOAD_LIMIT))
+{}
 
 bool RequestGroupMan::downloadFinished()
 {
@@ -274,7 +279,7 @@ void RequestGroupMan::configureRequestGroup
   const std::string& uriSelectorValue = _option->get(PREF_URI_SELECTOR);
   if(uriSelectorValue == V_FEEDBACK) {
     requestGroup->setURISelector
-      (SharedHandle<URISelector>(new ServerStatURISelector(_serverStatMan)));
+      (SharedHandle<URISelector>(new FeedbackURISelector(_serverStatMan)));
   } else if(uriSelectorValue == V_INORDER) {
     requestGroup->setURISelector
       (SharedHandle<URISelector>(new InOrderURISelector()));
@@ -313,7 +318,8 @@ void RequestGroupMan::fillRequestGroupFromReserver(DownloadEngine* e)
       configureRequestGroup(groupToAdd);
       Commands commands;
       createInitialCommand(groupToAdd, commands, e,
-			   _option->getAsBool(PREF_USE_HEAD));
+			   _option->getAsBool(PREF_USE_HEAD)||
+			   _option->getAsBool(PREF_DRY_RUN));
       _requestGroups.push_back(groupToAdd);
       ++count;
       e->addCommand(commands);
@@ -503,22 +509,45 @@ std::string RequestGroupMan::formatDownloadResult(const std::string& status, con
   return o.str();
 }
 
-bool RequestGroupMan::isSameFileBeingDownloaded(RequestGroup* requestGroup) const
+template<typename StringInputIterator, typename FileEntryInputIterator>
+static bool sameFilePathExists(StringInputIterator sfirst,
+			       StringInputIterator slast,
+			       FileEntryInputIterator ffirst,
+			       FileEntryInputIterator flast)
 {
-  // TODO it may be good to use dedicated method rather than use isPreLocalFileCheckEnabled
-  if(!requestGroup->isPreLocalFileCheckEnabled()) {
-    return false;
-  }
-  for(RequestGroups::const_iterator itr = _requestGroups.begin();
-      itr != _requestGroups.end(); ++itr) {
-    if((*itr).get() != requestGroup &&
-       (*itr)->getFilePath() == requestGroup->getFilePath()) {
+  for(; ffirst != flast; ++ffirst) {
+    if(std::binary_search(sfirst, slast, (*ffirst)->getPath())) {
       return true;
     }
   }
   return false;
 }
 
+bool RequestGroupMan::isSameFileBeingDownloaded(RequestGroup* requestGroup) const
+{
+  // TODO it may be good to use dedicated method rather than use
+  // isPreLocalFileCheckEnabled
+  if(!requestGroup->isPreLocalFileCheckEnabled()) {
+    return false;
+  }
+  std::deque<std::string> files;
+  for(RequestGroups::const_iterator itr = _requestGroups.begin();
+      itr != _requestGroups.end(); ++itr) {
+    if((*itr).get() != requestGroup) {
+      std::deque<SharedHandle<FileEntry> > entries =
+	(*itr)->getDownloadContext()->getFileEntries();
+      std::transform(entries.begin(), entries.end(),
+		     std::back_inserter(files),
+		     mem_fun_sh(&FileEntry::getPath));
+    }
+  }
+  std::sort(files.begin(), files.end());
+  std::deque<SharedHandle<FileEntry> > entries =
+    requestGroup->getDownloadContext()->getFileEntries();
+  return sameFilePathExists(files.begin(), files.end(),
+			    entries.begin(), entries.end());
+}
+
 void RequestGroupMan::halt()
 {
   for(RequestGroups::const_iterator itr = _requestGroups.begin();
@@ -612,4 +641,16 @@ void RequestGroupMan::removeStaleServerStat(time_t timeout)
   _serverStatMan->removeStaleServerStat(timeout);
 }
 
+bool RequestGroupMan::doesOverallDownloadSpeedExceed()
+{
+  return _maxOverallDownloadSpeedLimit > 0 &&
+    _maxOverallDownloadSpeedLimit < calculateStat().getDownloadSpeed();
+}
+
+bool RequestGroupMan::doesOverallUploadSpeedExceed()
+{
+  return _maxOverallUploadSpeedLimit > 0 &&
+    _maxOverallUploadSpeedLimit < calculateStat().getUploadSpeed();
+}
+
 } // namespace aria2
