@@ -32,7 +32,6 @@ public:
     adaptor.reset(new MultiDiskAdaptor());
     adaptor->setPieceLength(2);
     adaptor->setStoreDir(".");
-    adaptor->setTopDir(".");
   }
 
   void testWriteData();
@@ -54,16 +53,22 @@ std::deque<SharedHandle<FileEntry> > createEntries() {
     SharedHandle<FileEntry>(new FileEntry("file3.txt", 0, 22)),
     SharedHandle<FileEntry>(new FileEntry("file4.txt", 2, 22)),
     SharedHandle<FileEntry>(new FileEntry("file5.txt", 0, 24)),
+    SharedHandle<FileEntry>(new FileEntry("file6.txt", 3, 24)),
+    SharedHandle<FileEntry>(new FileEntry("file7.txt", 0, 27)),
+    SharedHandle<FileEntry>(new FileEntry("file8.txt", 2, 27)),
   };
-  //           1    1    2    2
-  // 0....5....0....5....0....5
-  // ++--++--++--++--++--++--++
+  //           1    1    2    2    3
+  // 0....5....0....5....0....5....0
+  // ++--++--++--++--++--++--++--++--
   // | file0
   // *************** file1
   //                ******* file2
   //                       | file3
   //                       ** flie4
   //                         | file5
+  //                         *** file6
+  //                            |file7
+  //                            ** file8
   std::deque<SharedHandle<FileEntry> > entries(&array[0],
 					       &array[arrayLength(array)]);
   for(std::deque<SharedHandle<FileEntry> >::const_iterator i = entries.begin();
@@ -192,7 +197,9 @@ void MultiDiskAdaptorTest::testResetDiskWriterEntries()
   }
   {
     std::deque<SharedHandle<FileEntry> > fileEntries = createEntries();
-    fileEntries[5]->setRequested(false);
+    for(size_t i = 5; i < 9; ++i) {
+      fileEntries[i]->setRequested(false);
+    }
     adaptor->setFileEntries(fileEntries);
     // In openFile(), resetDiskWriterEntries() are called.
     adaptor->openFile();
@@ -210,7 +217,7 @@ void MultiDiskAdaptorTest::testResetDiskWriterEntries()
   }
   {
     std::deque<SharedHandle<FileEntry> > fileEntries = createEntries();
-    for(size_t i = 1; i < 6; ++i) {
+    for(size_t i = 1; i < 9; ++i) {
       fileEntries[i]->setRequested(false);
     }
     adaptor->setFileEntries(fileEntries);
@@ -228,7 +235,7 @@ void MultiDiskAdaptorTest::testResetDiskWriterEntries()
   }
   {
     std::deque<SharedHandle<FileEntry> > fileEntries = createEntries();
-    for(size_t i = 2; i < 6; ++i) {
+    for(size_t i = 2; i < 9; ++i) {
       fileEntries[i]->setRequested(false);
     }
     adaptor->setFileEntries(fileEntries);
@@ -246,6 +253,29 @@ void MultiDiskAdaptorTest::testResetDiskWriterEntries()
 
     adaptor->closeFile();
   }
+  {
+    std::deque<SharedHandle<FileEntry> > fileEntries = createEntries();
+    for(size_t i = 0; i < 6; ++i) {
+      fileEntries[i]->setRequested(false);
+    }
+    fileEntries[8]->setRequested(false);
+    adaptor->setFileEntries(fileEntries);
+    adaptor->openFile();
+    std::deque<SharedHandle<DiskWriterEntry> > entries =
+      adaptor->getDiskWriterEntries();
+    CPPUNIT_ASSERT(entries[0]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(entries[1]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(entries[2]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(entries[3]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(entries[4]->getDiskWriter().isNull());
+    // entries[6] spans entries[5] in the current implementation.
+    CPPUNIT_ASSERT(!entries[5]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(!entries[6]->getDiskWriter().isNull());
+    CPPUNIT_ASSERT(!entries[7]->getDiskWriter().isNull());
+    // entries[6] spans entries[8]
+    CPPUNIT_ASSERT(!entries[8]->getDiskWriter().isNull());
+    adaptor->closeFile();
+  }
 }
 
 void readFile(const std::string& filename, char* buf, int bufLength) {
@@ -342,23 +372,19 @@ void MultiDiskAdaptorTest::testReadData() {
 void MultiDiskAdaptorTest::testCutTrailingGarbage()
 {
   std::string dir = "/tmp";
-  std::string topDir = ".";
-  std::string topDirPath = dir+"/"+topDir;
   std::string prefix = "aria2_MultiDiskAdaptorTest_testCutTrailingGarbage_";
   SharedHandle<FileEntry> entries[] = {
-    SharedHandle<FileEntry>(new FileEntry(prefix+"1", 256, 0)),
-    SharedHandle<FileEntry>(new FileEntry(prefix+"2", 512, 256))
+    SharedHandle<FileEntry>(new FileEntry(dir+"/"+prefix+"1", 256, 0)),
+    SharedHandle<FileEntry>(new FileEntry(dir+"/"+prefix+"2", 512, 256))
   };
   for(size_t i = 0; i < arrayLength(entries); ++i) {
-    createFile(topDirPath+"/"+entries[i]->getPath(),
-	       entries[i]->getLength()+100);
+    createFile(entries[i]->getPath(), entries[i]->getLength()+100);
   }
   std::deque<SharedHandle<FileEntry> > fileEntries
     (&entries[0], &entries[arrayLength(entries)]);
   
   MultiDiskAdaptor adaptor;
   adaptor.setStoreDir(dir);
-  adaptor.setTopDir(topDir);
   adaptor.setFileEntries(fileEntries);
   adaptor.setMaxOpenFiles(1);
   adaptor.setPieceLength(1);
@@ -368,30 +394,27 @@ void MultiDiskAdaptorTest::testCutTrailingGarbage()
   adaptor.cutTrailingGarbage();
 
   CPPUNIT_ASSERT_EQUAL((uint64_t)256,
-		       File(topDirPath+"/"+entries[0]->getPath()).size());
+		       File(entries[0]->getPath()).size());
   CPPUNIT_ASSERT_EQUAL((uint64_t)512,
-		       File(topDirPath+"/"+entries[1]->getPath()).size());
+		       File(entries[1]->getPath()).size());
 }
 
 void MultiDiskAdaptorTest::testSize()
 {
   std::string dir = "/tmp";
-  std::string topDir = ".";
-  std::string topDirPath = dir+"/"+topDir;
   std::string prefix = "aria2_MultiDiskAdaptorTest_testSize_";
   SharedHandle<FileEntry> entries[] = {
-    SharedHandle<FileEntry>(new FileEntry(prefix+"1", 1, 0)),
-    SharedHandle<FileEntry>(new FileEntry(prefix+"2", 1, 1))
+    SharedHandle<FileEntry>(new FileEntry(dir+"/"+prefix+"1", 1, 0)),
+    SharedHandle<FileEntry>(new FileEntry(dir+"/"+prefix+"2", 1, 1))
   };
   for(size_t i = 0; i < arrayLength(entries); ++i) {
-    createFile(topDirPath+"/"+entries[i]->getPath(), entries[i]->getLength());
+    createFile(entries[i]->getPath(), entries[i]->getLength());
   }
   std::deque<SharedHandle<FileEntry> > fileEntries
     (&entries[0], &entries[arrayLength(entries)]);
   
   MultiDiskAdaptor adaptor;
   adaptor.setStoreDir(dir);
-  adaptor.setTopDir(topDir);
   adaptor.setFileEntries(fileEntries);
   adaptor.setMaxOpenFiles(1);
   adaptor.setPieceLength(1);
@@ -403,20 +426,18 @@ void MultiDiskAdaptorTest::testSize()
 
 void MultiDiskAdaptorTest::testUtime()
 {
-  std::string storeDir = "/tmp";
-  std::string topDir = "aria2_MultiDiskAdaptorTest_testUtime";
-  std::string prefix = storeDir+"/"+topDir;
+  std::string storeDir = "/tmp/aria2_MultiDiskAdaptorTest_testUtime";
   SharedHandle<FileEntry> entries[] = {
-    SharedHandle<FileEntry>(new FileEntry("requested", 0, 0)),
-    SharedHandle<FileEntry>(new FileEntry("notFound", 0, 0)),
-    SharedHandle<FileEntry>(new FileEntry("notRequested", 0, 0)),
-    SharedHandle<FileEntry>(new FileEntry("anotherRequested", 0, 0)),
+    SharedHandle<FileEntry>(new FileEntry(storeDir+"/requested", 0, 0)),
+    SharedHandle<FileEntry>(new FileEntry(storeDir+"/notFound", 0, 0)),
+    SharedHandle<FileEntry>(new FileEntry(storeDir+"/notRequested", 0, 0)),
+    SharedHandle<FileEntry>(new FileEntry(storeDir+"/anotherRequested", 0, 0)),
   };
 
-  createFile(prefix+"/"+entries[0]->getPath(), entries[0]->getLength());
-  File(prefix+"/"+entries[1]->getPath()).remove();
-  createFile(prefix+"/"+entries[2]->getPath(), entries[2]->getLength());
-  createFile(prefix+"/"+entries[3]->getPath(), entries[3]->getLength());
+  createFile(entries[0]->getPath(), entries[0]->getLength());
+  File(entries[1]->getPath()).remove();
+  createFile(entries[2]->getPath(), entries[2]->getLength());
+  createFile(entries[3]->getPath(), entries[3]->getLength());
 
   entries[2]->setRequested(false);
 
@@ -424,7 +445,6 @@ void MultiDiskAdaptorTest::testUtime()
     (&entries[0], &entries[arrayLength(entries)]);
   MultiDiskAdaptor adaptor;
   adaptor.setStoreDir(storeDir);
-  adaptor.setTopDir(topDir);
   adaptor.setFileEntries(fileEntries);
 
   time_t atime = (time_t) 100000;
@@ -433,15 +453,13 @@ void MultiDiskAdaptorTest::testUtime()
   CPPUNIT_ASSERT_EQUAL((size_t)2, adaptor.utime(Time(atime), Time(mtime)));
   
   CPPUNIT_ASSERT_EQUAL((time_t)mtime,
-		       File(prefix+"/"+entries[0]->getPath())
-		       .getModifiedTime().getTime());
+		       File(entries[0]->getPath()).getModifiedTime().getTime());
 
   CPPUNIT_ASSERT_EQUAL((time_t)mtime,
-		       File(prefix+"/"+entries[3]->getPath())
-		       .getModifiedTime().getTime());
+		       File(entries[3]->getPath()).getModifiedTime().getTime());
 
-  CPPUNIT_ASSERT((time_t)mtime != File(prefix+"/"+entries[2]->getPath())
-		 .getModifiedTime().getTime());
+  CPPUNIT_ASSERT((time_t)mtime !=
+		 File(entries[2]->getPath()).getModifiedTime().getTime());
 }
 
 } // namespace aria2
